﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.Controls.DragDrop;
using Telerik.Windows.Controls.TreeView;
using Telerik.Windows;
using System.Collections.ObjectModel;
using Telerik.Windows.Controls;
using System.Collections;

namespace ListBoxToTreeView
{
	public partial class MainPage : UserControl
	{
		public MainPage()
		{
			InitializeComponent();

			this.radTreeView.ItemsSource = GenerateData();
			this.listBox.ItemsSource = GenerateEmployees();


			this.listBox.AddHandler(RadDragAndDropManager.DragQueryEvent, new EventHandler<DragDropQueryEventArgs>(OnListBoxDragQuery));
			this.radTreeView.AddHandler(RadDragAndDropManager.DropInfoEvent, new EventHandler<DragDropEventArgs>(OnTreeViewDropInfo));

		}

		#region data generation
		private ObservableCollection<Company> GenerateData()
		{
			ObservableCollection<Company> result = new ObservableCollection<Company>();

			for (int i = 0; i < 2; i++)
			{
				Company c = new Company()
				{
					Name = String.Format("Company {0}", i)
				};

				for (int j = 0; j < 2; j++)
				{
					Department d = new Department()
					{
						Name = String.Format("Department {0}", j)
					};

					for (int k = 0; k < 2; k++)
					{
						Employee e = new Employee()
						{
							Name = String.Format("Employee {0}", k)
						};
						d.Employees.Add(e);
					}
					c.Departments.Add(d);
				}
				result.Add(c);
			}
			return result;
		}

		private ObservableCollection<ListBoxData> GenerateEmployees()
		{
			ObservableCollection<ListBoxData> result = new ObservableCollection<ListBoxData>();

			for (int i = 0; i < 10; i++)
			{
				ListBoxData item = new ListBoxData()
					{
						EmployeeName = String.Format("Employee {0}", i),
						IconPath = "images/RedIcon.png"
					};
				result.Add(item);
			}
			return result;
		}
		#endregion

		#region DragDrop event handlers

		//ListBox OnDragQuery event handler
		private void OnListBoxDragQuery(object sender, DragDropQueryEventArgs e)
		{
			if (e.Options.Status == DragStatus.DragQuery)
			{
				e.QueryResult = true;
				e.Handled = true;
				var payload = listBox.SelectedItems.Cast<object>().ToList();
				e.Options.Payload = payload;
				//Here we generate the DragCue. Technically we should generate it in the
				//DragInfo event with Status DragInProgress. This way we keep the
				//presentation and a possible business logic separate. We can do it in the
				//DragQuery event when we are not interested in this separation.
				e.Options.ArrowCue = RadDragAndDropManager.GenerateArrowCue();
				var cue = new TreeViewDragCue();
				//Please note that we cannot set *any* source since the items may be
				//visual objects which cannot be displayed twice. This method tries to
				//find their non-visual believable representations.
				cue.SetSafeItemsSource(payload);
				cue.ItemTemplate = listBox.ItemTemplate;
				cue.IsDropPossible = false;
				//e.Options.DragCue = cue;
			}
			if (e.Options.Status == DragStatus.DropSourceQuery)
			{
				e.QueryResult = true;
				e.Handled = true;
			}
		}

		//treeView OnDropInfo event handler
		private void OnTreeViewDropInfo(object sender, DragDropEventArgs e)
		{
			RadTreeViewItem destinationItem = e.Options.Destination as RadTreeViewItem;
			DropPosition dropPosition = destinationItem.DropPosition;
			int dropIndex = destinationItem.Index;

			Telerik.Windows.Controls.ItemsControl parentControl = Telerik.Windows.Controls.ItemsControl.ItemsControlFromItemContainer(destinationItem);

			if (e.Options.Status == DragStatus.DropComplete && destinationItem.Item is Employee)
			{
				if (dropPosition == DropPosition.Before)
				{

					foreach (var draggedItem in e.Options.Payload as IEnumerable)
					{
						Employee emp = new Employee();
						emp.Name = (draggedItem as ListBoxData).EmployeeName;
						(parentControl.ItemsSource as IList).Insert(dropIndex, emp);
						destinationItem.DefaultImageSrc = (draggedItem as ListBoxData).IconPath;
					}
				}
				else
				{
					foreach (var draggedItem in e.Options.Payload as IEnumerable)
					{
						Employee emp = new Employee();
						emp.Name = (draggedItem as ListBoxData).EmployeeName;
						(parentControl.ItemsSource as IList).Insert(dropIndex + 1, emp);
						destinationItem.DefaultImageSrc = (draggedItem as ListBoxData).IconPath;
					}
				}
			}
		}
		#endregion
	}

	public class ListBoxData
	{
		public string EmployeeName { get; set; }
		public string IconPath { get; set; }
	}
}

